@@ -81,6 +81,8 @@ class GithubDataSource(ToucanDataSource):
         title='Entities Limit',
         description='Max Number of entities such as teams and repositories to extract',
     )
+    _oauth_trigger = 'instance'
+    oauth2_version = Field('1', **{'ui.hidden': True})
 
     @classmethod
     def get_form(cls, connector: 'GithubConnector', current_config, **kwargs):
@@ -97,6 +99,7 @@ class GithubConnector(ToucanConnector):
     _auth_flow = 'oauth2'
     auth_flow_id: Optional[str]
     data_source_model: GithubDataSource
+    _oauth_trigger = 'instance'
 
     @staticmethod
     def get_connector_secrets_form() -> ConnectorSecretsForm:
@@ -407,6 +410,7 @@ class GithubConnector(ToucanConnector):
         permissions: Optional[dict] = None,
         offset: int = 0,
         limit: Optional[int] = None,
+        get_row_count: Optional[bool] = False,
     ) -> DataSlice:
         """
         Method to retrieve a part of the data as a pandas dataframe
